@@ -15,10 +15,7 @@
 /* Name of the program which is called to generate missing pk files */
 #define MAKETEXPK "mktexpk"
 
-#define ERR_STREAM stdout   /* ???? */
-
-#include <stdlib.h>
-#include <stdarg.h>
+#define ERR_STREAM stderr   /* ???? */
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -26,14 +23,17 @@
 #include <inttypes.h>
 #endif
 
+/* For some reason FreeBSD does not follow the C standard */
+/* It has int32_t but not INT32_MAX                       */
+#ifndef INT32_MIN
+#define INT32_MIN   (-2147483647-1)
+#endif
+#ifndef INT32_MAX
+#define INT32_MAX   (2147483647)
+#endif
+              
 #include <gd.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h> // open/close
-
-#include <ctype.h> // isprint
-
 #ifdef HAVE_LIBKPATHSEA
 #include <kpathsea/kpathsea.h>
 #endif
@@ -46,10 +46,10 @@ typedef  int     bool;
 #ifndef HAVE_VPRINTF
 # ifdef HAVE_DOPRNT
 #  define   vfprintf(stream, message, args)  _doprnt(message, args, stream)
+# else
+#  define vprintf vprintf_and_doprnt_missing
+   /* If we have neither, should fall back to fprintf with fixed args.  */
 # endif
-#else
-# define vprintf vprintf_and_doprnt_missing
-  /* If we have neither, should fall back to fprintf with fixed args.  */
 #endif
 
 /*************************************************************/
@@ -63,7 +63,10 @@ typedef  int     bool;
 /*#define PIXROUND(x,c) ((((double)x+(double)(c>>1))/(double)c)+0.5)*/
 /*#define PIXROUND(x,c) (((x)+c)/(c))*/
 /*#define PIXROUND(x,c) ((x+c-1)/(c))*/
-#define PIXROUND(x,c) ((x)/(c))
+/*#define PIXROUND(x,c) ((x)/(c))*/
+/* integer round to the nearest number, not towards zero */
+#define PIXROUND(num,den) ((num)>0 ? ((num)+(den)/2)/(den) : -(((den)/2-(num))/(den)))
+
 
 /*************************************************************************/
 
@@ -228,12 +231,13 @@ void      CreateImage(void);
 void      DrawCommand(unsigned char*, void* /* dvi/vf */); 
 void      DrawPages(void);
 void      WriteImage(char*, int);
-void      LoadAChar(int32_t c, register struct pk_char * ptr);
+void      LoadAChar(int32_t, register struct pk_char *);
 int32_t   SetChar(int32_t);
 int32_t   SetPK(int32_t,int32_t, int32_t);
 int32_t   SetVF(int32_t);
 int32_t   SetRule(int32_t, int32_t, int32_t, int32_t, int);
-
+void      BeginVFMacro(struct font_entry*);
+void      EndVFMacro(void);
 
 /**************************************************/
 void handlepapersize(char*,int*,int*);
@@ -319,6 +323,12 @@ EXTERN struct timeb timebuffer;
 
 EXTERN int   resolution INIT(300);
 EXTERN char *MFMODE     INIT("cx");
+#ifdef HAVE_GDIMAGEPNGEX
+EXTERN int   compression INIT(1);
+#endif
+#ifdef  HAVE_GDIMAGECREATETRUECOLOR
+EXTERN bool truecolor INIT(FALSE);
+#endif
 # define  max(x,y)       if ((y)>(x)) x = y
 # define  min(x,y)       if ((y)<(x)) x = y
 
