@@ -1,7 +1,10 @@
 #include "dvipng.h"
+#include <fcntl.h> // open/close
 #include <sys/mman.h>
 #include <sys/stat.h>
 
+#define PROPER_OVERSTRIKE
+
 #define PK_POST 245
 #define PK_PRE 247
 #define PK_ID 89
@@ -10,38 +13,76 @@ unsigned char   dyn_f;
 int             repeatcount;
 int             poshalf;
 
-void    LoadAChar(int32_t, register struct pk_char *);
-
-int32_t SetPK(int32_t c, int32_t h,int32_t v)
+int32_t SetPK(int32_t c, int32_t hh,int32_t vv)
 {
   register struct pk_char *ptr = currentfont->chr[c];
                                       /* temporary pk_char pointer */
   int red,green,blue;
   int Color,i;
-
   /*
    * Draw character. Remember now, we have stored the different
    * greyscales in glyph.data with darkest last.  Draw the character
    * greyscale by greyscale, lightest first.
    */
+#ifdef PROPER_OVERSTRIKE 
+  int x,y;
+  int pos=0;
+  int bgColor,pixelcolor;
+  hh -= ptr->xOffset/shrinkfactor;
+  vv -= ptr->yOffset/shrinkfactor;
+
   for( i=1; i<=ptr->glyph.nchars ; i++) {
-    red = bRed-(bRed-Red)*i/shrinkfactor/shrinkfactor;
-    green = bGreen-(bGreen-Green)*i/shrinkfactor/shrinkfactor;
-    blue = bBlue-(bBlue-Blue)*i/shrinkfactor/shrinkfactor;
+    red = bRed-(bRed-Red)*i/ptr->glyph.nchars;
+    green = bGreen-(bGreen-Green)*i/ptr->glyph.nchars;
+    blue = bBlue-(bBlue-Blue)*i/ptr->glyph.nchars;
+    Color = gdImageColorResolve(page_imagep,red,green,blue);
+    for( y=0; y<ptr->glyph.h; y++) {
+      for( x=0; x<ptr->glyph.w; x++) {
+	if (ptr->glyph.data[pos]>0) {
+	  bgColor = gdImageGetPixel(page_imagep, hh + x, vv + y);
+	  if (bgColor > 0) {
+	    red=gdImageRed(page_imagep, bgColor);
+	    green=gdImageGreen(page_imagep, bgColor);
+	    blue=gdImageBlue(page_imagep, bgColor);
+	    red = red-(red-Red)*i/ptr->glyph.nchars;
+	    green = green-(green-Green)*i/ptr->glyph.nchars;
+	    blue = blue-(blue-Blue)*i/ptr->glyph.nchars;
+	    pixelcolor = gdImageColorResolve(page_imagep, red, green, blue);
+	    gdImageSetPixel(page_imagep, hh + x, vv + y, pixelcolor);
+	  } else
+	    gdImageSetPixel(page_imagep, hh + x, vv + y, Color);
+	}
+	pos++;
+      }
+    }
+  }
+#else /* not PROPER_OVERSTRIKE */  
+  for( i=1; i<=ptr->glyph.nchars ; i++) {
+    red = bRed-(bRed-Red)*i/ptr->glyph.nchars;
+    green = bGreen-(bGreen-Green)*i/ptr->glyph.nchars;
+    blue = bBlue-(bBlue-Blue)*i/ptr->glyph.nchars;
     Color = gdImageColorResolve(page_imagep,red,green,blue);
     gdImageChar(page_imagep, &(ptr->glyph),
-		PIXROUND(h, dvi->conv*shrinkfactor)
-		-PIXROUND(ptr->xOffset,shrinkfactor)
-		+x_offset,
-		PIXROUND(v, dvi->conv*shrinkfactor)
-		-PIXROUND(ptr->yOffset,shrinkfactor)
-		+y_offset,
+		hh - PIXROUND(ptr->xOffset,shrinkfactor),
+		vv - PIXROUND(ptr->yOffset,shrinkfactor),
 		i,Color);
   }
+#endif /* PROPER_OVERSTRIKE */
   return(ptr->tfmw);
 }
 
 
+#if 0 /* HAVE_GDIMAGECOLORRESOLVEALPHA */
+    /* Only works on truecolor images. */
+    Color = gdImageColorResolveAlpha(page_imagep,Red,Green,Blue,
+				   128-128*i/shrinkfactor/shrinkfactor);
+    gdImageChar(page_imagep, &(ptr->glyph),
+		hh - PIXROUND(ptr->xOffset,shrinkfactor),
+		vv - PIXROUND(ptr->yOffset,shrinkfactor),
+		i,Color);
+#endif /* HAVE_GDIMAGECOLORRESOLVEALPHA */
+
+
 unsigned char getnyb(unsigned char** pos)
 {
   if (poshalf == 0) {
@@ -328,7 +369,6 @@ void LoadAChar(int32_t c, register struct pk_char * ptr)
 #endif
   /*
     Separate the different greyscales with the darkest last.
-    See SetChar in set.c
   */
   for (j = 0; j < shrunk_height; j++) {	
     for (i = 0; i < shrunk_width; i++) {    
